/*
===============================================================================

  FILE:  SVreader_sva.h
  
  CONTENTS:
  
    Reads a tetrahedral volume mesh from a Streaming Mesh ASCII format (SMA).
  
  PROGRAMMERS:
  
    martin isenburg@cs.unc.edu
  
  COPYRIGHT:
  
    copyright (C) 2005  martin isenburg@cs.unc.edu
    
    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  
  CHANGE HISTORY:
  
    07 December 2009 -- made it handle .hex format (number of cells and verts)
    13 August 2008 -- extended to hex meshes during the morning paper session
    20 February 2005 -- created after streaming points turned out too tricky
  
===============================================================================
*/
#ifndef SVREADER_SMA_H
#define SVREADER_SMA_H

#include "svreader.h"

#include <stdio.h>

class SVreader_sva : public SVreader
{
public:

  // SVreader interface function implementations

  SVevent read_element();
  SVevent read_event();

  void close(bool close_file=true);

  // SVreader_sva functions

  bool open(FILE* file);

  SVreader_sva();
  ~SVreader_sva();

private:
  FILE* file;
  int skipped_lines;
  char* line;
  int have_finalized, next_finalized;
  I32* finalized_vertices;
  int format;
};

#endif
