/*
===============================================================================

  FILE:  svreadwithboundarymarkers.h
  
  CONTENTS:
  
    A filter for reading a volume streaming mesh with all boundary vertices
    marked. What the filter really checks for is whether each face incident
    to a vertex appears in *two* tetrahedra. At first we do a faster test by
    checking whether the vertex is surrounded by an odd number of tetrahedra.

  PROGRAMMERS:
  
    martin isenburg@cs.unc.edu
  
  COPYRIGHT:
  
    copyright (C) 2007  martin isenburg@cs.unc.edu
    
    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  
  CHANGE HISTORY:
  
    07 October 2007 -- adapted from the similar streaming mesh version
  
===============================================================================
*/
#ifndef SVREAD_WITH_BOUNDARY_MARKERS_H
#define SVREAD_WITH_BOUNDARY_MARKERS_H

#include "svreaddereferenced.h"

class SVreadWithBoundaryMarkers : public SVreadDereferenced
{
public:

  // additional vertex variables

  bool v_boundary;

  // additional cell variables

  bool* c_boundary;

  // SVreadDereferenced interface function implementations

  SVevent read_element();
  SVevent read_event();

  void close(bool close_file=true);

  // SVreadWithBoundaryMarkers functions

  bool open(SVreader* svreader);

  SVreadWithBoundaryMarkers();
  ~SVreadWithBoundaryMarkers();

private:
  SVreader* svreader;
  int num_non_pre_order;

  int have_new, next_new;
  int new_vertices[4];
  int have_cell;
  int have_finalized, next_finalized;
  int finalized_vertices[4];

  int read_input();
  int read_cell();
};

#endif
