/*
===============================================================================

  FILE:  svreadwithoutpoorelements.h
  
  CONTENTS:
  
    A filter for reading a streaming volume mesh without those tetrahedral
    elements whose ideal weight inverse mean ratio is above the cut_off (and
    also without vertices that have lost all their tetrahedra).

  PROGRAMMERS:
  
    martin isenburg@cs.unc.edu
  
  COPYRIGHT:
  
    copyright (C) 2007  martin isenburg@cs.unc.edu
    
    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  
  CHANGE HISTORY:
  
    17 October 2007 -- created after making a sunny-side up breakfast for henna

===============================================================================
*/
#ifndef SVREAD_WITHOUT_POOR_ELEMENTS_H
#define SVREAD_WITHOUT_POOR_ELEMENTS_H

#include "svreaddereferenced.h"

class SVreadWithoutPoorElements : public SVreadDereferenced
{
public:

  // SVreadDereferenced interface function implementations

  SVevent read_element();
  SVevent read_event();

  void close(bool close_file=true);

  // SVreadWithoutPoorElements functions

  bool open(SVreader* svreader, float cut_off = 100.0f);

  SVreadWithoutPoorElements();
  ~SVreadWithoutPoorElements();

private:
  SVreader* svreader;
  int num_non_pre_order;
  float cut_off;

  int have_new, next_new;
  int new_vertices[4];
  int have_cell;
  int have_finalized, next_finalized;
  int finalized_vertices[4];

  int read_input();
  int read_cell();
};

#endif
