/*
===============================================================================

  FILE:  svreader.cpp
  
  CONTENTS:
  
    see corresponding header file
  
  PROGRAMMERS:
  
    martin isenburg@cs.unc.edu
  
  COPYRIGHT:
  
    copyright (C) 2007  martin isenburg@cs.unc.edu
    
    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  
  CHANGE HISTORY:
  
    see corresponding header file
  
===============================================================================
*/
#include "svreader.h"

#include <stdio.h>
#include <stdlib.h>

#include "vecnfv.h"

bool SVreader::compute_bounding_box()
{
  SVevent event;

  if (v_pnum_f == 0)
  {
    fprintf(stderr, "WARNING: cannot compute bounding box because v_pnum_f == 0\n");
    return false;
  }

  if (v_pmin_f) delete [] v_pmin_f;
  if (v_pmax_f) delete [] v_pmax_f;
  v_pmin_f = new float[v_pnum_f];
  v_pmax_f = new float[v_pnum_f];

  if (c_pnum_f)
  {
    if (c_pmin_f) delete [] c_pmin_f;
    if (c_pmax_f) delete [] c_pmax_f;
    c_pmin_f = new float[c_pnum_f];
    c_pmax_f = new float[c_pnum_f];

    while ((event = read_element()) > SV_EOF)
    {
      if (event == SV_VERTEX)
      {
        if (v_count == 1)
        {
          VecCopyNfv(v_pmin_f, v_prop_f, v_pnum_f);
          VecCopyNfv(v_pmax_f, v_prop_f, v_pnum_f);
        }
        else
        {
          VecUpdateMinMaxNfv(v_pmin_f, v_pmax_f, v_prop_f, v_pnum_f);
        }
      }
      else if (event == SV_TETRAHEDRON || event == SV_HEXAHEDRON)
      {
        if (c_count == 1)
        {
          VecCopyNfv(c_pmin_f, c_prop_f, c_pnum_f);
          VecCopyNfv(c_pmax_f, c_prop_f, c_pnum_f);
        }
        else
        {
          VecUpdateMinMaxNfv(c_pmin_f, c_pmax_f, c_prop_f, c_pnum_f);
        }
      }
    }
  }
  else
  {
    while ((event = read_element()) > SV_EOF)
    {
      if (event == SV_VERTEX)
      {
        if (v_count == 1)
        {
          VecCopyNfv(v_pmin_f, v_prop_f, v_pnum_f);
          VecCopyNfv(v_pmax_f, v_prop_f, v_pnum_f);
        }
        else
        {
          VecUpdateMinMaxNfv(v_pmin_f, v_pmax_f, v_prop_f, v_pnum_f);
        }
      }
    }
  }

  return true;
}

void SVreader::close(bool close_file)
{
  fprintf(stderr, "WARNING: classes derived from SVreader should implement their own close() function\n");
}

SVreader::SVreader()
{
  // init of SVreader interface
  v_idx = -1;

  v_pnum_f = 0;
  v_pmin_f = 0;
  v_pmax_f = 0;
  v_prop_f = 0;

  c_pnum_f = 0;
  c_pmin_f = 0;
  c_pmax_f = 0;
  c_prop_f = 0;

  c_nverts = -1;
  c_idx = 0;
  c_final = 0;

  final_idx = -1;

  ncomments = 0;
  comments = 0;

  ncells = -1;
  nverts = -1;

  c_count = -1;
  v_count = -1;

  post_order = false;
}

SVreader::~SVreader()
{
  // clean-up for SVreader interface
  if (v_count != -1)
  {
    close(); // user must have forgotten to close the mesh
  }
  if (comments)
  {
    for (int i = 0; i < ncomments; i++)
    {
      free(comments[i]);
    }
    free(comments);
  }
  if (v_pmin_f) delete [] v_pmin_f;
  if (v_pmax_f) delete [] v_pmax_f;
  if (v_prop_f) delete [] v_prop_f;
  if (c_pmin_f) delete [] c_pmin_f;
  if (c_pmax_f) delete [] c_pmax_f;
  if (c_prop_f) delete [] c_prop_f;
}
